<%@ Application Language="C#" %>

<script runat="server">

    void Application_Start(object sender, EventArgs e) {
        // Code that runs on application startup
        ScrewTurn.Wiki.StartupTools.Startup();
        
    }
    
    void Application_End(object sender, EventArgs e) {
        //  Code that runs on application shutdown
        ScrewTurn.Wiki.StartupTools.Shutdown();
    }

    void Application_BeginRequest(object sender, EventArgs e) {
        // Extract the physical page name, i.e. MainPage.ashx
        string pageName = System.IO.Path.GetFileNameWithoutExtension(HttpContext.Current.Request.PhysicalPath);
        // Exctract the extension, i.e. .ashx
        string ext = System.IO.Path.GetExtension(HttpContext.Current.Request.PhysicalPath).ToLower();
        
        if (pageName + ext == "NotifyNewUserRegistration.ashx")
            return;
        
        // Remove trailing dot, .ashx -> ashx
        if(ext.Length > 0) ext = ext.Substring(1);
        string qs = "";
        try {
            // This might throw exceptions if 3rd party modules interefer with the request pipeline
            qs = HttpContext.Current.Request.Url.Query.Replace("?", "&");
        }
        catch { }
        if(ext.Equals("ashx")) {
            // Demand the request to Default.aspx
            HttpContext.Current.RewritePath("~/Default.aspx?Page=" + pageName + qs);
        }
    }  
        
    void Application_Error(object sender, EventArgs e) {
        // Code that runs when an unhandled error occurs
        Exception ex = Server.GetLastError();
        if(ex is HttpException) {
            // Tries to redirect an inexistent .aspx page to a probably existent .ashx page
            HttpException hx = (HttpException)ex;
            if(hx.GetHttpCode() == 404) {
                if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                {
                    string page = Request["Page"] ??  ScrewTurn.Wiki.Settings.DefaultPage;
                    Response.Redirect("Default.aspx?Page=" + ScrewTurn.Wiki.Tools.UrlEncode(page));
                }
                else
                {
                    string page = System.IO.Path.GetFileNameWithoutExtension(Request.PhysicalPath);
                    Response.Redirect(page + ScrewTurn.Wiki.Settings.PageExtension);
                }     
                return;
            }
        }
        LogError(Server.GetLastError());
        Response.Redirect("Error.aspx");
    }

    private void LogError(Exception ex) {
        if(ex.InnerException != null) ex = ex.InnerException;
        ScrewTurn.Wiki.Log.LogEntry(HttpContext.Current.Request.Url.ToString() + "<br />" + ex.Source + " thrown " + ex.GetType().ToString() + "<br />" + ex.Message.Replace("\r", "").Replace("\n", "<br />") + "<br />" + ex.StackTrace.Replace("\r", "").Replace("\n", "<br />"), ScrewTurn.Wiki.EntryType.Error, "SYSTEM");
    }   

    void Session_Start(object sender, EventArgs e) {
        // Code that runs when a new session is started
        ScrewTurn.Wiki.Users.OnlineUsers++;
        ScrewTurn.Wiki.SessionFacade.Breadcrumbs = new ScrewTurn.Wiki.BreadcrumbsManager();
    }

    void Session_End(object sender, EventArgs e) {
        // Code that runs when a session ends. 
        // Note: The Session_End event is raised only when the sessionstate mode
        // is set to InProc in the Web.config file. If session mode is set to StateServer 
        // or SQLServer, the event is not raised.
        ScrewTurn.Wiki.Users.OnlineUsers--;
    }
       
</script>
