
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using ScrewTurn.Wiki.PluginFramework;

namespace ScrewTurn.Wiki {

	public partial class Login : BasePage {

        protected void Page_Load(object sender, EventArgs e) {
            if (!IsPostBack) SerialNumber1.Create();

            Page.Title = Resources.Messages.LoginTitle + " - " + Settings.WikiTitle;

			lblResult.Text = "";
			lblResult.CssClass = "";

			lblLostPassword.Text = Resources.Messages.LostPassword;

			if(SessionFacade.LoginKey == null) {
                if(Request.Cookies[Settings.LoginCookieName] != null) {
                    string key = Request.Cookies[Settings.LoginCookieName].Values["LoginKey"];
                    if(Tools.ComputeSecuredUsernameHash("admin").Equals(key)) {
						SessionFacade.LoginKey = key;
						SessionFacade.Username = "admin";
						SessionFacade.Admin = true;
                        Log.LogEntry("Admin logged in through cookie", EntryType.General, "SYSTEM");
						if(Request["Redirect"] != null) Response.Redirect(Request["Redirect"]);
                    }
					Users u = Users.Instance;
                    bool found = false;
					for(int i = 0; i < u.AllUsers.Count; i++) {
						if(u.AllUsers[i].Active && Tools.ComputeSecuredUsernameHash(u.AllUsers[i].Username).Equals(key)) {
							SessionFacade.LoginKey = key;
							SessionFacade.Username = u.AllUsers[i].Username;
							if(u.AllUsers[i].Admin) {
								SessionFacade.Admin = true;
							}
							else {
								SessionFacade.Admin = false;
							}
							found = true;
							Log.LogEntry("User " + SessionFacade.Username + " logged in through cookie", EntryType.General, "SYSTEM");
							break;
						}
					}
                    if(!found) {
                        // Delete invalid cookie
                        HttpCookie cookie = Request.Cookies[Settings.LoginCookieName];
                        cookie.Expires = DateTime.Now.AddYears(-1);
                        Response.Cookies.Add(cookie);
                    }
                    else {
						if(Request["Redirect"] != null) {
							Response.Redirect(Request["Redirect"]);
						}
                    }
                }
            }
			if(SessionFacade.LoginKey != null) {
				mlvLogin.ActiveViewIndex = 0;
				lblLogout.Text = "<b>" + SessionFacade.Username + "</b>, " + lblLogout.Text;
			}
			else {
				if(Request["PasswordReset"] != null) mlvLogin.ActiveViewIndex = 2;
				else mlvLogin.ActiveViewIndex = 1;
			}
            if(Request["Activate"] != null && !Page.IsPostBack) {
				Users u = Users.Instance;
                for(int i = 0; i < u.AllUsers.Count; i++) {
                    if(Tools.ComputeSecuredUsernameHash(u.AllUsers[i].Username).Equals(Request["Activate"])) {
                        Log.LogEntry("Account activation requested for " + u.AllUsers[i].Username, EntryType.General, "SYSTEM");
                        if(u.AllUsers[i].Active) {
							lblResult.CssClass = "resultok";
							lblResult.Text = "<br /><br />" + Resources.Messages.AccountAlreadyActive;
                            return;
                        }
                        if(u.AllUsers[i].DateTime.AddHours(24).CompareTo(DateTime.Now) < 0) {
                            // Too late
							lblResult.CssClass = "resulterror";
							lblResult.Text = "<br /><br />" + Resources.Messages.AccountNotFound;
                            // Delete user (is this correct?)
                            u.RemoveUser(u.AllUsers[i]);
                            return;
                        }
                        // Activate User
						u.SetActivationStatus(u.AllUsers[i], true);
						lblResult.CssClass = "resultok";
						lblResult.Text = "<br /><br />" + Resources.Messages.AccountActivated;
                        return;
                    }
                }
				lblResult.CssClass = "resulterror";
				lblResult.Text = "<br /><br />" + Resources.Messages.AccountNotActivated;
                return;
            }
        }

        protected void btnLogin_Click(object sender, EventArgs e) {

            if (!SerialNumber1.CheckSN(txtSerialNumber.Text))
            {
				lblResult.CssClass = "resulterror";
				lblResult.Text = "<br /><br />֤!";
                return;
            }

			txtUsername.Style.Clear();
			txtPassword.Style.Clear();

            if(txtUsername.Text.Equals("admin") && txtPassword.Text.Equals(Settings.MasterPassword)) {
                if(chkRemember.Checked) {
                    HttpCookie cookie = new HttpCookie(Settings.LoginCookieName);
                    cookie.Expires = DateTime.Now.AddYears(100);
					cookie.Path = Settings.CookiePath;
                    cookie.Values.Add("LoginKey", Tools.ComputeSecuredUsernameHash(txtUsername.Text));
                    Response.Cookies.Add(cookie);
                }
				SessionFacade.LoginKey = Tools.ComputeSecuredUsernameHash(txtUsername.Text);
				SessionFacade.Username = txtUsername.Text;
				SessionFacade.Admin = true;
                Log.LogEntry("Admin logged in", EntryType.General, "SYSTEM");

				if(Request["Redirect"] != null) {
					Response.Redirect(Request["Redirect"]);
				}
				else {
                    Response.Redirect("Default.aspx");
				}
                return;
            }
			Users u = Users.Instance;

			UserInfo info = Users.Instance.Find(txtUsername.Text);
			if(info != null && info.Active && info.Provider.TestAccount(info, txtPassword.Text)) {
				if(chkRemember.Checked) {
					HttpCookie cookie = new HttpCookie(Settings.LoginCookieName);
					cookie.Expires = DateTime.Now.AddYears(100);
					cookie.Path = Settings.CookiePath;
					cookie.Values.Add("LoginKey", Tools.ComputeSecuredUsernameHash(txtUsername.Text));
					Response.Cookies.Add(cookie);
				}
				SessionFacade.LoginKey = Tools.ComputeSecuredUsernameHash(txtUsername.Text);
				SessionFacade.Username = txtUsername.Text;
				if(info.Admin) SessionFacade.Admin = true;
				else SessionFacade.Admin = false;
				Log.LogEntry("User " + SessionFacade.Username + " logged in", EntryType.General, "SYSTEM");

				if(Request["Redirect"] != null) {
					Response.Redirect(Request["Redirect"]);
				}
				else {
                    Response.Redirect("Default.aspx");
				}
			}
			else {
				lblResult.CssClass = "resulterror";
				lblResult.Text = "<br /><br />" + Resources.Messages.WrongUsernamePassword;
			}

        }

        protected void btnLogout_Click(object sender, EventArgs e) {
            if(Request.Cookies[Settings.LoginCookieName] != null) {
                HttpCookie cookie = Request.Cookies[Settings.LoginCookieName];
                cookie.Expires = DateTime.Now.AddDays(-7);
				cookie.Path = Settings.CookiePath;
                Response.Cookies.Add(cookie);
            }
            Session.Abandon();
			Log.LogEntry("User " + SessionFacade.Username + " logged out", EntryType.General, "SYSTEM");
            Response.Redirect("Login.aspx");
        }

        protected void btnResetPassword_Click(object sender, EventArgs e) {
			Users users = Users.Instance;
            UserInfo u = users.Find(txtUsernameReset.Text);
            if(u == null) {
            }
            else {
				if(u.Email.Equals(txtEmailReset.Text)) {
					users.ResetPassword(u);
					Log.LogEntry("Password reset requested for " + txtUsernameReset.Text, EntryType.General, "SYSTEM");
				}
				else {
				}
            }
			lblResult.CssClass = "resultok";
			lblResult.Text = "<br /><br />" + Resources.Messages.PasswordReset;
			txtUsernameReset.Text = "";
			txtEmailReset.Text = "";
        }

    }

}
