
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using ScrewTurn.Wiki.PluginFramework;

namespace ScrewTurn.Wiki {

	public partial class RSS : BasePage {

        protected void Page_Load(object sender, EventArgs e) {
			// Check Private Wiki
			bool hasAccess = true;
			if(SessionFacade.LoginKey == null && Settings.PrivateAccess) {
				// Look for username/password in the query string
				if(Request["Username"] != null && Request["Password"] != null) {
					// Try to authenticate
					UserInfo u = Users.Instance.Find(Request["Username"]);
					if(u != null) {
						// Very "dirty" way - pages should not access Providers
						if(u.Provider.TestAccount(u, Request["Password"])) {
							// Valid account
							hasAccess = true;
						}
						else hasAccess = false;
					}
					else {
						// Check for built-in admin account
						if(Request["Username"].Equals("admin") && Request["Password"].Equals(Settings.MasterPassword)) {
							hasAccess = true;
						}
						else hasAccess = false;
					}
				}
				else hasAccess = false;
			}

			if(!hasAccess) {
				// Return an empty page
				Response.Clear();
				Response.End();
				return;
			}

            Response.ClearContent();
            Response.ContentType = "text/xml;charset=UTF-8";
            Response.ContentEncoding = System.Text.Encoding.UTF8;

			Pages pages = Pages.Instance;

            if(Request["Page"] != null) {
				PageInfo page = pages.FindPage(Request["Page"]);
				if(page == null) return;
				PageContent content = Content.GetPageContent(page, true);
				if(Request["Discuss"] == null) {
					Response.Write(RssHead);

                    if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                    {
					    Response.Write("<channel>" + BuildChannelHead(Settings.WikiTitle + " - " + content.Title, Settings.MainUrl + "?Page=" + Tools.UrlEncode(page.Name), content.Title + " - " + Resources.Messages.PageUpdates));
                    }
                    else
                    {
					    Response.Write("<channel>" + BuildChannelHead(Settings.WikiTitle + " - " + content.Title, Settings.MainUrl + Tools.UrlEncode(page.Name) + Settings.PageExtension, content.Title + " - " + Resources.Messages.PageUpdates));
                    }
					Response.Write("<item>");
					Response.Write("<title><![CDATA[" + content.Title + "]]></title>");

                    if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                    {
					    Response.Write("<link>" + Settings.MainUrl + "?Page=" + Tools.UrlEncode(page.Name) + "</link>");
                    }
                    else
                    {
					    Response.Write("<link>" + Settings.MainUrl + Tools.UrlEncode(page.Name) + Settings.PageExtension + "</link>");
                    }
					Response.Write("<description><![CDATA[" + content.Title + ": " + Resources.Messages.ThePageHasBeenUpdatedBy + " " + content.User + (content.Comment.Length > 0 ? ".<br />" + content.Comment : ".") + "]]></description>");
					Response.Write("<pubDate>" + content.LastModified.ToUniversalTime().ToString("R") + "</pubDate>");

                    if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                    {
					    Response.Write("<guid>" + Settings.MainUrl + "?Page=" + Tools.UrlEncode(page.Name) + "</guid>");
                    }
                    else
                    {
					    Response.Write("<guid>" + Settings.MainUrl + Tools.UrlEncode(page.Name) + Settings.PageExtension + "</guid>");
                    }
					Response.Write("</item>");
					Response.Write("</channel>");

					Response.Write(RssFoot);
				}
				else {
					List<Message> messages = new List<Message>(Pages.Instance.GetPageMessages(page));
					// Un-tree Messages
					UnTreeMessages(messages, messages);
					// Sort from newer to older
					messages.Sort(new MessageDateTimeComparer(true));

					Response.Write(RssHead);

                    if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                    {
					    Response.Write("<channel>" + BuildChannelHead(Settings.WikiTitle + " - " + content.Title + " - Discussion Updates", Settings.MainUrl + "?Page=" + Tools.UrlEncode(page.Name) + "&Discuss=1", Settings.WikiTitle + " - " + content.Title + " - Discussion Updates"));
                    }
                    else
                    {
					    Response.Write("<channel>" + BuildChannelHead(Settings.WikiTitle + " - " + content.Title + " - Discussion Updates", Settings.MainUrl + Tools.UrlEncode(page.Name) + Settings.PageExtension + "?Discuss=1", Settings.WikiTitle + " - " + content.Title + " - Discussion Updates"));
                    }

					int shift;
					for(int i = 0; i < messages.Count; i++) {
						Response.Write("<item>");
						Response.Write("<title><![CDATA[" + messages[i].Subject + "]]></title>");
                        if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                        {
						    Response.Write("<link>" + Settings.MainUrl + "?Page=" + Tools.UrlEncode(page.Name) + "&Discuss=1</link>");
                        }
                        else
                        {
						    Response.Write("<link>" + Settings.MainUrl + Tools.UrlEncode(page.Name) + Settings.PageExtension + "?Discuss=1</link>");
                        }
						Response.Write("<description><![CDATA[" + Formatter.Format(messages[i].Body, null) + "]]></description>");
						Response.Write("<pubDate>" + messages[i].DateTime.ToUniversalTime().ToString("R") + "</pubDate>");
						Response.Write("<guid>" + Tools.UrlEncode(page.Name) + "." + messages[i].ID.ToString() + "</guid>");
						Response.Write("</item>");
					}

					Response.Write("</channel>");

					Response.Write(RssFoot);
				}
            }
			else {
				// All updates
				Response.Write(RssHead);

				Response.Write("<channel>" + BuildChannelHead(Settings.WikiTitle + " - " + Resources.Messages.PageUpdates, Settings.MainUrl, Resources.Messages.RecentPageUpdates));

				bool useCat = false;
				string cat = "";
				if(Request["Category"] != null) {
					useCat = true;
					cat = Request["Category"];
				}

				RecentChange[] ch = RecentChanges.Instance.AllChanges;
				Array.Reverse(ch);
				int shift;
				for(int i = 0; i < ch.Length; i++) {
					PageInfo p = Pages.Instance.FindPage(ch[i].Page);
					if(p != null && useCat) {
						CategoryInfo[] infos = Pages.Instance.GetCategoriesPerPage(p);
						if(infos.Length == 0) {
							if(!cat.Equals("-")) continue;
						}
						else {
							bool found = false;
							for(int k = 0; k < infos.Length; k++) {
								if(infos[k].Name.Equals(cat)) {
									found = true;
									break;
								}
							}
							if(!found) continue;
						}
					}
					else if(p == null && useCat) continue;
					Response.Write("<item>");
					Response.Write("<title><![CDATA[" + ch[i].Title + "]]></title>");
					if(ch[i].Change != Change.PageDeleted)
                    {
                        if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                        {
						    Response.Write("<link>" + Settings.MainUrl + "?Page=" + ch[i].Page + "</link>");
                        }
                        else
                        {
						    Response.Write("<link>" + Settings.MainUrl + ch[i].Page + Settings.PageExtension + "</link>");
                        }
                    }
					else Response.Write("<link>" + Settings.MainUrl + "</link>");
					Response.Write("<description><![CDATA[");
					switch(ch[i].Change) {
						case Change.PageUpdated:
							Response.Write(Resources.Messages.ThePageHasBeenUpdatedBy);
							break;
						case Change.PageDeleted:
							Response.Write(Resources.Messages.ThePageHasBeenDeletedBy);
							break;
						case Change.PageRenamed:
							Response.Write(Resources.Messages.ThePageHasBeenRenamedBy);
							break;
						case Change.PageRolledBack:
							Response.Write(Resources.Messages.ThePageHasBeenRolledBackBy);
							break;
					}
					Response.Write(" " + ch[i].User + (ch[i].Description.Length > 0 ? ".<br />" + ch[i].Description : ".") + "]]></description>");
					Response.Write("<pubDate>" + ch[i].DateTime.ToUniversalTime().ToString("R") + "</pubDate>");
                    if (System.Configuration.ConfigurationManager.AppSettings["DisableUrlRewrite"] == "true")
                    {
					    Response.Write("<guid>" + Settings.MainUrl + "?Page=" + ch[i].Page + "</guid>");
                    }
                    else
                    {
					    Response.Write("<guid>" + Settings.MainUrl + ch[i].Page + Settings.PageExtension + "</guid>");
                    }
					Response.Write("</item>");
				}
				Response.Write("</channel>");

				Response.Write(RssFoot);
			}

        }

		private void UnTreeMessages(List<Message> root, List<Message> messages) {
			for(int i = 0; i < messages.Count; i++) {
				root.AddRange(messages[i].Replies);
				UnTreeMessages(root, messages[i].Replies);
				messages[i].Replies.Clear();
			}
		}

		private string BuildChannelHead(string title, string link, string description) {
            string result = "";
            result += "<title><![CDATA[" + title + "]]></title>";
            result += "<link>" + link + "</link>";
            result += "<description>" + description + "</description>";
			DateTime now = DateTime.Now;
			result += "<pubDate>" + now.ToUniversalTime().ToString("R") + "</pubDate>";
            result += "<generator>ScrewTurn Wiki RSS Feed Generator</generator>";
            return result;
        }

        private string RssHead {
            get {
                string head = @"<rss version=""2.0"">" + "\r\n";
                return head;
            }
        }

        private string RssFoot {
            get {
                string foot = "</rss>" + "\r\n";
                return foot;
            }
        }

    }

}
