
using System;
using System.Data;
using System.Drawing;
using System.Configuration;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Configuration;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using ScrewTurn.Wiki.PluginFramework;
using System.Collections.Generic;

namespace ScrewTurn.Wiki {

	public partial class Register : BasePage {
		
		#region Initialization

        protected void Page_Load(object sender, EventArgs e) {
            if (!string.IsNullOrEmpty(System.Configuration.ConfigurationManager.AppSettings["UserRegisterRedirectPage"]))
            {
                Response.Redirect(System.Configuration.ConfigurationManager.AppSettings["UserRegisterRedirectPage"]);
            }

			if(Request["Captcha"] != null) {
				Response.ContentType = "image/jpeg";
				Response.Clear();

				string s = SessionFacade.CaptchaString;

				Font f = new Font("Times New Roman", 30, FontStyle.Italic | FontStyle.Bold, GraphicsUnit.Pixel);
				StringFormat format = new StringFormat();
				format.Alignment = StringAlignment.Center;
				format.LineAlignment = StringAlignment.Center;

				Bitmap bmp = new Bitmap(150, 40, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
				Graphics g = Graphics.FromImage(bmp);
				g.Clear(Color.White);
				g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
				g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;

				g.DrawString(s, f, Brushes.Black, new RectangleF(0, 0, bmp.Width, bmp.Height), format);

				for(int i = -2; i < bmp.Width / 10; i++) {
					g.DrawLine(Pens.OrangeRed, i * 10, 0, i * 10 + 20, bmp.Height);
				}
				for(int i = -2; i < bmp.Width / 10 + 10; i++) {
					g.DrawLine(Pens.Blue, i * 10, 0, i * 10 - 60, bmp.Height);
				}

				g.Dispose();

				bmp.Save(Response.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);

				return;
			}

			// Test whehter the default Users Provider is read-only
			IUsersStorageProvider p = Collectors.UsersProviderCollector.GetProvider(Settings.DefaultUsersProvider);
			if(p.ReadOnly) {
				Log.LogEntry("Default Users Provider (" + p.Information.Name + ") is read-only, aborting Account Creation", EntryType.Warning, "SYSTEM");
				Response.Redirect("Error.aspx");
			}

            Page.Title = Resources.Messages.RegisterTitle + " - " + Settings.WikiTitle;

			if(!Settings.UsersCanRegister) {
				if((SessionFacade.LoginKey != null && !SessionFacade.Admin) || SessionFacade.LoginKey == null)
					Response.Redirect("AccessDenied.aspx");
			}

			switch(Settings.AccountActivationMode.ToLower()) {
				case "email":
					lblAccountActivationMode.Text = Resources.Messages.ActivationEmail;
					break;
				case "admin":
					lblAccountActivationMode.Text = Resources.Messages.ActivationAdmin;
					break;
				case "auto":
					lblAccountActivationMode.Text = Resources.Messages.ActivationAuto;
					break;
			}

            //if(Settings.DisableCaptchaControl) {
            //    lblCaptcha.Visible = false;
            //    txtCaptcha.Visible = false;
            //    imgCaptcha.Visible = false;
            //    rfvCaptcha.Enabled = false;
            //    cvCaptcha.Enabled = false;
            //}

			if(!Page.IsPostBack) {
				// Generate captcha string
				Random r = new Random();
				string c = "";
				c += (char)r.Next(49, 58); // 1 - 9 (not 0)
				c += (char)r.Next(65, 79); // A - N (not O)
				c += (char)r.Next(97, 111); // a - n (not o)
				c += (char)r.Next(49, 58); // 1 - 9 (not 0)
				c += (char)r.Next(80, 91); // P - Z
				c += (char)r.Next(112, 123); // p - z
				SessionFacade.CaptchaString = c;
				//imgCaptcha.ImageUrl = "Register.aspx?Captcha=1";

				LoadRegExp();
				LoadValidationLocalizations();

                SerialNumber1.Create();
			}

		}

		private void LoadRegExp() {
			// Load RegExp's from Web.Config

			//User 
			rxvUserName.ValidationExpression = Settings.UsernameRegex;
			rxvEmail1.ValidationExpression = Settings.EmailRegex;
			rxvPassword1.ValidationExpression = Settings.PasswordRegex;
		}

		private void LoadValidationLocalizations() {
			//User validators
			rfvUsername.ErrorMessage = Resources.Messages.RequiredField;
			rfvUsername.ToolTip = Resources.Messages.RequiredField;
			rfvEmail1.ErrorMessage = Resources.Messages.RequiredField;
			rfvEmail1.ToolTip = Resources.Messages.RequiredField;
			rfvEmail2.ErrorMessage = Resources.Messages.RequiredField;
			rfvEmail2.ToolTip = Resources.Messages.RequiredField;
			rfvPassword1.ErrorMessage = Resources.Messages.RequiredField;
			rfvPassword1.ToolTip = Resources.Messages.RequiredField;
			rfvPassword2.ErrorMessage = Resources.Messages.RequiredField;
			rfvPassword2.ToolTip = Resources.Messages.RequiredField;
			rfvCaptcha.ErrorMessage = Resources.Messages.RequiredField;
			rfvCaptcha.ToolTip = Resources.Messages.RequiredField;

			rxvUserName.ErrorMessage = Resources.Messages.InvalidUsername;
			rxvUserName.ToolTip = Resources.Messages.InvalidUsername;
			rxvEmail1.ErrorMessage = Resources.Messages.InvalidEmail;
			rxvEmail1.ToolTip = Resources.Messages.InvalidEmail;
			rxvPassword1.ErrorMessage = Resources.Messages.InvalidPassword;
			rxvPassword1.ToolTip = Resources.Messages.InvalidPassword;

			cvUsername.ErrorMessage = Resources.Messages.UsernameAlreadyExists;
			cvUsername.ToolTip = Resources.Messages.UsernameAlreadyExists;
            //cvEmail1.ErrorMessage = Resources.Messages.EmailsNotEqual;
            //cvEmail1.ToolTip = Resources.Messages.EmailsNotEqual;
            //cvEmail2.ErrorMessage = Resources.Messages.EmailsNotEqual;
            //cvEmail2.ToolTip = Resources.Messages.EmailsNotEqual;

            //cvPassword1.ErrorMessage = Resources.Messages.PasswordsNotEqual;
            //cvPassword1.ToolTip = Resources.Messages.PasswordsNotEqual;
            //cvPassword2.ErrorMessage = Resources.Messages.PasswordsNotEqual;
            //cvPassword2.ToolTip = Resources.Messages.PasswordsNotEqual;

			//cvCaptcha.ErrorMessage = Resources.Messages.WrongControlText;
			//cvCaptcha.ToolTip = Resources.Messages.WrongControlText;
		}
		
		#endregion

		protected void btnRegister_Click(object sender, EventArgs e) {
			if (!Settings.UsersCanRegister)
            {
                return;
            }

			lblResult.Text = "";
			lblResult.CssClass = "";

            if (!SerialNumber1.CheckSN(txtCaptcha.Text))
            {
                ltCheckSNError.Text = "<font color=red>֤!</font>";
                return;
            }
            else
            {
                ltCheckSNError.Text = string.Empty;
            }

            if (txtEmail1.Text != txtEmail2.Text)
            {
                ltEmailEqualError.Text = "<font color=red>Emailһ!</font>";
                return;
            }
            else
            {
                ltEmailEqualError.Text = string.Empty;
            }

            if (txtPassword1.Text != txtPassword2.Text)
            {
                ltPasswordEqualError.Text = "<font color=red>벻һ!</font>";
                return;
            }
            else
            {
                ltPasswordEqualError.Text = string.Empty;
            }

			if(!Page.IsValid) return;

			// Ready to save the user
			Users u = Users.Instance;
			Log.LogEntry("Account creation requested for " + txtUsername.Text, EntryType.General, "SYSTEM");
			Users.Instance.AddUser(txtUsername.Text, txtPassword1.Text, txtEmail1.Text, Settings.AccountActivationMode.ToLower().Equals("auto"), false, null);
			if(Settings.AccountActivationMode.ToLower().Equals("email")) {
				string body = Tools.LoadFile(Settings.AccountActivationMessageFile);
				body = body.Replace("##WIKITITLE##", Settings.WikiTitle).Replace("##USERNAME##", txtUsername.Text).Replace("##EMAILADDRESS##", Settings.ContactEmail);
				body = body.Replace("##ACTIVATIONLINK##", Settings.MainUrl + "Login.aspx?Activate=" + Tools.ComputeSecuredUsernameHash(txtUsername.Text));
				EmailSender.Send(txtEmail1.Text, Settings.SenderEmail, "Account Activation - " + Settings.WikiTitle, body, false);
			}
			if(Settings.AccountActivationMode.ToLower().Equals("admin")) {
				List<UserInfo> admins = Users.Instance.Administrators;
				string[] rec = new string[admins.Count];
				for(int i = 0; i < rec.Length; i++) {
					rec[i] = admins[i].Email;
				}
				EmailSender.SendMassEmail(rec, Settings.SenderEmail, Settings.WikiTitle + " - User Account Approval", "The User Account <" + txtUsername.Text + "> requires approval. Please visit the Administration page to view the Account's details using the link below.\r\n\r\n" + Settings.MainUrl + "Admin.aspx?ActivationMode=1", false);
			}

			lblResult.CssClass = "resultok";
			lblResult.Text = "<br /><br />" + Resources.Messages.AccountCreated;
			btnRegister.Enabled = false;
			pnlRegister.Visible = false;
		}

		#region Custom Validators
		
		protected void cvUsername_ServerValidate(object source, ServerValidateEventArgs args) {
			if(txtUsername.Text.ToLower().Equals("admin") || txtUsername.Text.ToLower().Equals("guest")) {
				args.IsValid = false;
			}
			else {
				UserInfo u = Users.Instance.Find(txtUsername.Text);
				if(u != null) {
					args.IsValid = false;
				}
			}
		}
		
		protected void cvPassword1_ServerValidate(object source, ServerValidateEventArgs args) {
			args.IsValid = (string.Compare(txtPassword1.Text, txtPassword2.Text, true) == 0);
		}
		
		protected void cvPassword2_ServerValidate(object source, ServerValidateEventArgs args) {
			args.IsValid = true;
		}
		
		protected void cvEmail1_ServerValidate(object source, ServerValidateEventArgs args) {
			args.IsValid = (string.Compare(txtEmail1.Text, txtEmail2.Text, true) == 0);
		}
		
		protected void cvEmail2_ServerValidate(object source, ServerValidateEventArgs args) {
			args.IsValid = true;
		}
		
		protected void cvCaptcha_ServerValidate(object source, ServerValidateEventArgs args) {
			if (!Settings.DisableCaptchaControl) {
				args.IsValid = (string.Compare(txtCaptcha.Text, SessionFacade.CaptchaString, true) == 0);
			}
			else {
				args.IsValid = true;
			}
		}
		
		#endregion

	}

}
