
using System;
using System.Collections.Generic;
using System.Text;
using ScrewTurn.Wiki.PluginFramework;

namespace ScrewTurn.Wiki {

	/// <summary>
	/// Stores a Local UserInfo object.
	/// </summary>
	[Serializable]
	public class LocalUserInfo : UserInfo {

		private string passwordHash;

		/// <summary>
		/// Initializes a new instance of the <b>LocalUserInfo</b> class.
		/// </summary>
		/// <param name="username">The Username.</param>
		/// <param name="email">The Email.</param>
		/// <param name="active">Specifies whether the Account is active or not.</param>
		/// <param name="dateTime">The creation DateTime.</param>
		/// <param name="admin">Specifies whether the User is an Admin or not.</param>
		/// <param name="provider">The Users Storage Provider that manages the User.</param>
		/// <param name="passwordHash">The Password Hash.</param>
		public LocalUserInfo(string username, string email, bool active, DateTime dateTime, bool admin, IUsersStorageProvider provider, string passwordHash)
			: base(username, email, active, dateTime, admin, provider) {
			this.passwordHash = passwordHash;
		}

		/// <summary>
		/// Gets or sets the Password Hash.
		/// </summary>
		public string PasswordHash {
			get { return passwordHash; }
			set { passwordHash = value; }
		}

	}

}
