
using System;
using System.Collections.Generic;
using System.Text;

namespace ScrewTurn.Wiki.PluginFramework {

	/// <summary>
	/// The base interface that all the Providers must implement. All the Provider Type-specific interfaces inherit from this one or from a one, either directly or from a derived interface.
	/// </summary>
	/// <remarks>This interface should not be implemented directly by a class.</remarks>
	public interface IProvider {

		/// <summary>
		/// Initializes the Storage Provider.
		/// </summary>
		/// <param name="host">The Host of the Component.</param>
		/// <param name="config">The Configuration data, if any.</param>
		/// <remarks>If the configuration string is not valid, the methoud should throw an exception.</remarks>
		void Init(IHost host, string config);

		/// <summary>
		/// Method invoked on shutdown.
		/// </summary>
		/// <remarks>This method might not be invoked in some cases.</remarks>
		void Shutdown();

		/// <summary>
		/// Gets the Information about the Provider.
		/// </summary>
		ComponentInformation Information { get; }

	}

}
